 
#include <dht.h>
dht DHT;
#define DHT11_PIN 1

int luxMeterPin = 11; // the pin for the phototransistor 
int prevLight=0;
 
int period = 100;
int change, oldvalue, value;

int micPin = 8; // the pin for the mic module
int threshold = 800;


int red = 3; // the pin for red leds
int green = 5; // the pin for green leds
int blue = 6; // the pin for blue leds

int redValue = 0;
int greenValue = 0;
int blueValue = 0;

int values[] = {0,0,0};

int tempMultiplier; 

int modeButton = 7; // the pin for the button
bool buttonDown = false;
int mode = 1;

void setup() 
{
 pinMode(red, OUTPUT);
 pinMode(green, OUTPUT);
 pinMode(blue, OUTPUT);
 pinMode(modeButton, INPUT);
 //pinMode(micPin, INPUT);
 pinMode(luxMeterPin, INPUT);
 Serial.begin(9600);
 Serial.print("DHT11, \t");
 tempMultiplier = 255 /20;
}

void loop()
{
  
  switch(mode)
  {
    case 0:
     ButtonCheck(); //Turns all the leds off and only checks to see if the mode button is pressed
      break;
    case 1:
     RainbowPulse(); // Pulses red to green to blue
      break;
    case 2:
     Rainbow(); // Cycles through an rgb rainbow
      break;
    case 3:
     FakeHearthBeat(); // Pulsates red like a beating hearth
      break;
    case 4:
     Temperature(); // Turns blue when cold and red when hot
      break;
    case 5:
     LightIntencity(); // Lights up white the darker the enviroment gets
      break;
    case 6: 
     SoundVolume(); // Turns purple when near a load sound
       break;
    case 7:
     App(); // makes the leds controllable via the android app
      break;
  }
}

void Rainbow() // turns 1 color led to 255 while turning the previuos from 255 to 0 creating a smooth rainbow fade
{
  for(int i = 0; i < 3; i++)
  {
    for(int v = 0; v <= 255; v++)
    {
      values[i] = v;
      if(i > 0)
      {
        values[i-1] = 255 - v;
      }
      else
      {
        values[2] = 255 - v;
      }
      analogWrite(red, values[0]);
      analogWrite(green, values[1]);
      analogWrite(blue, values[2]);
      if(buttonDelay(10))
      {
        return;
      }
    }
  }
}

void RainbowPulse() // Pulsates red to green to blue 
{
  if(Pulse(red, 2))
  {
    return;
  }
  if(Pulse(green, 2))
  {
    return;
  }
  if(Pulse(blue, 2))
  {
    return;
  }
}

bool Pulse(int color, int updateDelay) // Pulsates a analoge signal from 0 to 255 back to 0 over a certain pin with the updateDelay as the interval
{
 for(int i =0; i < 256;i++)
 {
  analogWrite(color, i);
  if(buttonDelay(updateDelay))
  {
    return true;
  }
 }
 for(int i =255; i >= 0;i--)
 {
  analogWrite(color, i);
  if(buttonDelay(updateDelay))
  {
    return true;
  }
 }
 return false;
}

void FakeHearthBeat() // Pulses the red leds on and off on the 
{
  for(int i = 0; i < 255 / 2; i++)
  {
    analogWrite(red,i ); 
    if(buttonDelay(1))
    { 
      return;
    }
  }
  for(int i = 255/ 4; i >= 0; i--)
  {
    analogWrite(red,i);
    if(buttonDelay(1))
    { 
      return;
    }
  }
  if(buttonDelay(100))
    { 
      return;
    }
  for(int i = 0; i < 255 / 2; i++)
  {
    analogWrite(red,i );
    if(buttonDelay(1))
    { 
      return;
    }
  }
  for(int i = 255/ 4; i >= 0; i--)
  {
    analogWrite(red,i);
    if(buttonDelay(1))
    { 
      return;
    }
  }
  if(buttonDelay(600))
  { 
    return;
  }
}

void Temperature() // turns the leds blue when cold and red when hot
{
  int chk = DHT.read11(DHT11_PIN);
  analogWrite(blue, 255 - (DHT.temperature-10) * tempMultiplier);
  analogWrite(red,(DHT.temperature-10) * tempMultiplier);
  if(buttonDelay(200))
  {
    return;
  }
}

void LightIntencity() // Lights up brighter the darker the surroundings get 
{
  Serial.println(analogRead(0));
  int light = analogRead(0);
  for(int i = 0; i < 100; i++)
  {
   int lerpLight = lerp(prevLight, light, (float)i / (float)100);
   if(lerpLight >= 500)
   {
    int val = 0;
     if(lerpLight - 500 >255)
     {
      val = 255;
     }
     else
     {
      val = lerpLight - 500;
     }
     analogWrite(red, val);
     analogWrite(green, val);
     analogWrite(blue, val);
    }
    else
    {
     analogWrite(red, 0);
     analogWrite(green, 0);
     analogWrite(blue, 0);
    }
    if(buttonDelay(10))
    {
      return;
    }
  }
  prevLight = light;
}

void SoundVolume() // Lights up purple when near a loud sound, lights up solid purple when you talk ( if it is mounted near the colar of the dress)
{
  Serial.println(analogRead(2));
  int valA, valB;
  valA = analogRead(2);
  if(buttonDelay(1))
  {
    return;
  } 
  valB = analogRead(2);
  if(valB-valA > 1)
  {
    for(int i =0; i < 255;i+=255/2)
    {
     analogWrite(red, i);
      analogWrite(blue, i);
      if(buttonDelay(1))
      {
       return ;
      }
    }
    digitalWrite(red, HIGH);
    digitalWrite(blue, HIGH);
    if(buttonDelay(10))
    {
      return;
    } 
  }
  else
  {
    digitalWrite(red, LOW);
    digitalWrite(blue, LOW);
  }
  ButtonCheck();
}

float lerp(float a, float b, float x)
{ 
 return a + x * (b - a);
}

void App() // Decodes the incomming bluetooth data and colors the dress accordingly
{
   String data = "";
   int currIndex = 0;
   int rIndex, gIndex, bIndex, cIndex;
   bool done = false;

   if(Serial.available()>0){
    while(done == false) 
    {
      if(Serial.available()>0){
      char curr = Serial.read();
      data+=curr;
      switch(curr)
      {
        case 'r':
        rIndex = currIndex;
        Serial.println(rIndex);
        break;
        case 'g':
        gIndex = currIndex;
        Serial.println(gIndex);
        break;
        case 'b':
        bIndex = currIndex;
        Serial.println(bIndex);
        break; 
        case 'c':
        cIndex = currIndex;
        done = true;
        break;
      }
      currIndex++;
     } 
    }
   Serial.println(data);  
   String redstr = "";
   String greenstr = "";
   String bluestr = "";
   for(int i = rIndex; i < rIndex+(gIndex - rIndex)-1; i++)
   {
     redstr += data[i+1];
   }
   for(int i = gIndex; i < gIndex+(bIndex - gIndex)-1; i++)
   {
     greenstr += data[i+1];
   }
   for(int i = bIndex; i < bIndex+(cIndex - bIndex)-1; i++)
   {
     bluestr += data[i+1];
   }
   analogWrite(red, (float)255 * pow((float)redstr.toInt()/(float)255, 2));
   analogWrite(green, (float)255 * pow((float)greenstr.toInt()/(float)255, 2));
   analogWrite(blue, (float)255 * pow((float)bluestr.toInt()/(float)255, 2));
   }
   ButtonCheck();
}

bool buttonDelay(int milliSeconds) // checks the button and then delays for 1 ms for the input amount of time, this makes it possible to delay in script without losing the abillity to the read the button being pressed
{
  for(int i = 0; i < milliSeconds; i++)
  {
   if(ButtonCheck())
   {
    return true;
   }
   delay(1);
  }
  return false;
}

bool ButtonCheck() // Checks the button
{
  if(digitalRead(modeButton) == LOW && buttonDown == false)
  {
    buttonDown = true;
    if(mode < 7)
    {
      mode++;
    }
    else
    {
      mode = 0;
    } 
    analogWrite(red, 0);
    analogWrite(green, 0);
    analogWrite(blue, 0);
    Serial.println(mode);
    return true;
  }
  else if(digitalRead(modeButton) == HIGH && buttonDown == true)
  {
    buttonDown = false;
  }
  return false;
}






